!----------------------------------------------------------!
! Description: cOCR (OCRClass.Clw)
! Purpose    : Provides the ability to OCR .tif, .bmp and .jpg images
! Owner      : John Dunn
! Updated    : 06/05/2006 clarion mag version 1
!
!----------------------------------------------------------!

                                MEMBER

                                INCLUDE('OCRClass.Def'),ONCE
                                INCLUDE('OCRClass.Inc'),ONCE

                                MAP
                                  MODULE('OCRDLL.Lib')
AddImage                            PROCEDURE(LONG plImageSet,LONG plImage),SHORT,RAW,PASCAL
CreateMultipleImg                   PROCEDURE()LONG,RAW,PASCAL
DeskewImg                           PROCEDURE(LONG plImage),LONG,RAW,PASCAL
DIBToImg                            PROCEDURE(LONG plhDIB),LONG,RAW,PASCAL
EraseBlackBordersImg                PROCEDURE(LONG plImage),LONG,RAW,PASCAL
ExtractImgArea                      PROCEDURE(LONG plImage,LONG plXPos,LONG plYPos,LONG plWidth,LONG plHeight),LONG,RAW,PASCAL
FreeImg                             PROCEDURE(LONG plImage),RAW,PASCAL
FreeMultipleImg                     PROCEDURE(LONG plImageSet),RAW,PASCAL
GetNbImages                         PROCEDURE(LONG plImageSet),LONG,RAW,PASCAL
GetImage                            PROCEDURE(LONG plImageSet,LONG plIdx),LONG,RAW,PASCAL
GetImgRes                           PROCEDURE(LONG plImage,*LONG plHRes,*LONG plVRes),RAW,PASCAL
GetImgSize                          PROCEDURE(LONG plImage,*LONG plWidth,*LONG plHeight),RAW,PASCAL
LoadImg                             PROCEDURE(LONG plFilenameAddress),LONG,RAW,PASCAL
LoadMultipleImg                     PROCEDURE(LONG plFilenameAddress),LONG,RAW,PASCAL
OCR                                 PROCEDURE(LONG plImage,LONG plNoisy),LONG,RAW,PASCAL
OCRSetProgressHandler               PROCEDURE(LONG plProc),LONG,RAW,PASCAL
OCRSetOutputHandler                 PROCEDURE(LONG plProc),LONG,RAW,PASCAL
RotateImg                           PROCEDURE(LONG plImage,LONG plAngle),LONG,RAW,PASCAL
SaveImg                             PROCEDURE(LONG plFilenameAddress, LONG plImage),LONG,RAW,PASCAL
SaveMultipleImg                     PROCEDURE(LONG plFilenameAddress, LONG plImage),LONG,RAW,PASCAL
SetLanguage                         PROCEDURE(LONG plLanguage,LONG plDictDir),RAW,PASCAL
SetOutputMode                       PROCEDURE(LONG plMode),RAW,PASCAL
SetOutputWindow                     PROCEDURE(LONG plhWnd),RAW,PASCAL
                                  END ! module

                                  MODULE('API')
                                    GetModuleFileName(UNSIGNED,*CSTRING,LONG),ULONG,PASCAL,RAW,NAME('GetModuleFileNameA')
                                    OutputDebugString(*CSTRING),PASCAL,RAW,NAME('OutputDebugStringA')
                                  END ! module

OutputHandler                     PROCEDURE(LONG plInfotype,LONG plParam),PASCAL
ProgressHandler                   PROCEDURE(LONG plPercent),LONG,PASCAL

                                END ! map

                                OMIT('***',_C55_)
LEVEL:Benign                    EQUATE(0)
LEVEL:User                      EQUATE(1)
LEVEL:Program                   EQUATE(2)
LEVEL:Fatal                     EQUATE(3)
LEVEL:Cancel                    EQUATE(4)
LEVEL:Notify                    EQUATE(5)
                                ***

lProgressFEq                    LONG                       ! progress field equate for progress handler

szOutputText                    CSTRING(10001)             ! output string for output handler

!----------------------------------------------------------!
! Construct(),PRIVATE
!----------------------------------------------------------!

cOCR.Construct                  PROCEDURE()                !! ,PRIVATE

eMethodName                     EQUATE('.Construct ')

  CODE

  SELF.bTraceOn = True
  SELF.Trace(eMethodName)

  SELF.bTraceOn  = False

!----------------------------------------------------------!
! Destruct(),PRIVATE
!----------------------------------------------------------!

cOCR.Destruct                   PROCEDURE()                !! ,PRIVATE

eMethodName                     EQUATE('.Destruct ')

  CODE

  SELF.Trace(eMethodName)

!----------------------------------------------------------!
! GetLastErrNo PROCEDURE(),LONG
!
! Returns the Last Error Number
!----------------------------------------------------------!

cOCR.GetLastErrNo               PROCEDURE()                !! ,LONG

eMethodName                     EQUATE('.GetLastErrNo ')

  CODE

  SELF.Trace(eMethodName)
  SELF.Trace(eMethodName & 'Complete Returned = ' & SELF.lLastErrNo)

  RETURN SELF.lLastErrNo

!----------------------------------------------------------!
! GetLastError PROCEDURE(),STRING
!
! Returns the Last Error Message
!----------------------------------------------------------!

cOCR.GetLastError               PROCEDURE()                !! ,STRING

eMethodName                     EQUATE('.GetLastError ')

  CODE

  SELF.Trace(eMethodName)
  SELF.Trace(eMethodName & 'Complete Returned = ' & SELF.sLastErrMsg)

  RETURN SELF.sLastErrMsg

!----------------------------------------------------------!
! SetLastError PROCEDURE(LONG plLastErrNo,STRING psLastErrMsg),PRIVATE
!
! Sets the Last Error Number
!----------------------------------------------------------!

cOCR.SetLastError               PROCEDURE(LONG plLastErrNo,STRING psLastErrMsg) !! PRIVATE

eMethodName                     EQUATE('.SetLastError ')

  CODE

  SELF.Trace(eMethodName)

  SELF.lLastErrNo  = plLastErrNo
  SELF.sLastErrMsg = psLastErrMsg
  SELF.Trace(eMethodName & SELF.lLastErrNo & eOCRSpace & SELF.sLastErrMsg)
  SELF.Trace(eMethodName & 'Complete')

!----------------------------------------------------------!
! LoadImage PROCEDURE(STRING psInputFile),BYTE
!
! Loads a TIFF image
!----------------------------------------------------------!
  
cOCR.LoadImage                  PROCEDURE(STRING psInputFile) !! ,BYTE

eMethodName                     EQUATE('.LoadImage ')

bErrorLevel                     BYTE

  CODE

  SELF.Trace(eMethodName)

  bErrorLevel      = LEVEL:Benign
  SELF.szInputFile = CLIP(psInputFile)
  SELF.Trace(eMethodName & 'szInputFile = ' & SELF.szInputFile)

  SELF.lImg = LoadImg(ADDRESS(SELF.szInputFile))
  IF SELF.lImg = eOCRZero
    SELF.SetLastError(eLoadImageErr,eLoadImageErrDesc)
    bErrorLevel = LEVEL:Fatal
  END ! loop

  SELF.Trace(eMethodName & 'lImg = '              & SELF.lImg)
  SELF.Trace(eMethodName & 'Complete Returned = ' & bErrorLevel)

  RETURN bErrorLevel

!----------------------------------------------------------!
! LoadMultipleImages PROCEDURE(),BYTE
!
! Loads a set of images from a TIFF file.
!----------------------------------------------------------!
  
cOCR.LoadMultipleImages         PROCEDURE(STRING psInputFile) !! ,BYTE

eMethodName                     EQUATE('.LoadMultipleImages ')

bErrorLevel                     BYTE

  CODE

  SELF.Trace(eMethodName)

  bErrorLevel      = LEVEL:Benign
  SELF.szInputFile = CLIP(psInputFile)
  SELF.Trace(eMethodName & 'szInputFile = ' & SELF.szInputFile)

  SELF.lSet = LoadMultipleImg(ADDRESS(SELF.szInputFile))
  IF SELF.lSet = eOCRZero
    SELF.SetLastError(eLoadMultiImageErr,eLoadMultiImageErrDesc)
    SELF.bLoadSingleImage = True
    bErrorLevel = SELF.LoadImage(psInputFile)
  END ! if

  SELF.Trace(eMethodName & 'lSet = '              & SELF.lSet)
  SELF.Trace(eMethodName & 'Complete Returned = ' & bErrorLevel)

  RETURN bErrorLevel

!----------------------------------------------------------!
! DIBToImage PROCEDURE(LONG plhDIB),BYTE
!
! Loads a device independant bitmap
!----------------------------------------------------------!
  
cOCR.DIBToImage                 PROCEDURE(LONG plhDIB)     !! ,BYTE

eMethodName                     EQUATE('.DIBToImage ')

bErrorLevel                     BYTE

  CODE

  SELF.Trace(eMethodName)

  bErrorLevel      = LEVEL:Benign
  SELF.Trace(eMethodName & 'plhDIB = ' & plhDIB)

  SELF.lImg = DIBToImg(plhDIB)
  IF SELF.lImg = eOCRZero
    SELF.SetLastError(eDIBToImageErr,eDIBToImageErrDesc)
    bErrorLevel = LEVEL:Fatal
  END ! loop

  SELF.Trace(eMethodName & 'lImg = '              & SELF.lImg)
  SELF.Trace(eMethodName & 'Complete Returned = ' & bErrorLevel)

  RETURN bErrorLevel

!----------------------------------------------------------!
! DeskewImage PROCEDURE(),BYTE
!
! Deskews skewed images
!----------------------------------------------------------!
  
cOCR.DeskewImage                PROCEDURE()                !! ,BYTE

eMethodName                     EQUATE('.DeskewImage ')

bErrorLevel                     BYTE

lRetVal                         LONG

  CODE

  SELF.Trace(eMethodName)

  SELF.Trace(eMethodName & 'bDeskewImage = ' & SELF.bDeskewImage)

  IF SELF.bDeskewImage = True
    bErrorLevel = LEVEL:Benign
    lRetVal     = DeskewImg(SELF.lImg)
    IF lRetVal ~= eOCRZero
      SELF.SetLastError(eDeskewErr,eDeskewErrDesc)
      bErrorLevel = LEVEL:Notify
    END ! if
  END ! if

  SELF.Trace(eMethodName & 'lRetVal = '           & lRetVal)
  SELF.Trace(eMethodName & 'Complete Returned = ' & bErrorLevel)

  RETURN bErrorLevel

!----------------------------------------------------------!
! EraseBlackBorders PROCEDURE(),BYTE
!
! Erases the black borders around clipped images
!----------------------------------------------------------!
  
cOCR.EraseBlackBorders          PROCEDURE()                !! ,BYTE

eMethodName                     EQUATE('.EraseBlackBorders ')

bErrorLevel                     BYTE

lRetVal                         LONG

  CODE

  SELF.Trace(eMethodName)

  SELF.Trace(eMethodName & 'bEraseBlackBorders = ' & SELF.bEraseBlackBorders)

  IF SELF.bEraseBlackBorders = True
    bErrorLevel = LEVEL:Benign
    lRetVal     = EraseBlackBordersImg(SELF.lImg)
    IF lRetVal ~= eOCRZero
      SELF.SetLastError(eEraseBlackBordersErr,eEraseBlackBordersErrDesc)
      bErrorLevel = LEVEL:Notify
    END ! if  
  END ! if

  SELF.Trace(eMethodName & 'lRetVal = '           & lRetVal)
  SELF.Trace(eMethodName & 'Complete Returned = ' & bErrorLevel)

  RETURN bErrorLevel

!----------------------------------------------------------!
! FreeImage PROCEDURE()
!
! Frees an existing image
!----------------------------------------------------------!
  
cOCR.FreeImage                  PROCEDURE()

eMethodName                     EQUATE('.FreeImage ')

  CODE  

  SELF.Trace(eMethodName)

  FreeImg(SELF.lImg)
  SELF.lImg = eOCRZero

  SELF.Trace(eMethodName & 'Complete')

!----------------------------------------------------------!
! FreeMultipleImages PROCEDURE()
!
! Frees a set of images. All the images in the set are also freed.
!----------------------------------------------------------!
  
cOCR.FreeMultipleImages         PROCEDURE()

eMethodName                     EQUATE('.FreeMultipleImages ')

  CODE  

  SELF.Trace(eMethodName)

  FreeMultipleImg(SELF.lSet)
  SELF.lSet = eOCRZero
  SELF.lImg = eOCRZero

  SELF.Trace(eMethodName & 'Complete')

!----------------------------------------------------------!
! SetOutputMode PROCEDURE(LONG plMode)
!
! Selects the output mode for the OCR engine
!----------------------------------------------------------!
  
cOCR.SetOutputMode              PROCEDURE(LONG plMode)

eMethodName                     EQUATE('.SetOutputMode ')

  CODE

  SELF.Trace(eMethodName)

  SetOutputMode(plMode)

  SELF.Trace(eMethodName & 'Complete')

!----------------------------------------------------------!
! SetLanguage PROCEDURE()
!
! Selects the language for the dictionary
!----------------------------------------------------------!
  
cOCR.SetLanguage                PROCEDURE()

eMethodName                     EQUATE('.SetLanguage ')

szLanguage                      CSTRING(11)

sDictionaryFileName             STRING(255)

  CODE

  SELF.Trace(eMethodName)

  IF SELF.bLanguage = eOCRNil
    SELF.bLanguage = eEnglish
  END ! if

  SELF.Trace(eMethodName & 'bLanguage = ' & SELF.bLanguage)

  SELF.szDictDir = CLIP(SELF.sDictDir)

  IF SELF.szDictDir = eOCRNil
    SELF.szDictDir = CLIP(SELF.ProgramDir(True))
  END ! if

  SELF.Trace(eMethodName & 'szDictDir = ' & SELF.szDictDir)

  IF SELF.bLanguage ~= eOCRZero
    CASE SELF.bLanguage
    OF 1;szLanguage = 'English'
    OF 2;szLanguage = 'French'
    OF 3;szLanguage = 'Dutch'
    OF 4;szLanguage = 'EngUK'
    OF 5;szLanguage = 'Custom'
    END ! case

    sDictionaryFileName = CLIP(SELF.AddBackSlash(SELF.szDictDir)) & szLanguage & '.wdc'

    IF NOT EXISTS(sDictionaryFileName)
      MESSAGE('The dictionary file: ' & CLIP(sDictionaryFileName) & ' does not exist for '  & |
              'the selected language.||The dictionary setting will be set to <39>None<39> ' & |
              '||Please check your setting and dictionary files.','OCR Class',ICON:Exclamation)
      SELF.bLanguage = eOCRZero
    END ! if
  END ! if

  SetLanguage(SELF.bLanguage,ADDRESS(SELF.szDictDir))

  SELF.Trace(eMethodName & 'Complete')

!----------------------------------------------------------!
! OCR PROCEDURE(),BYTE
!
! Recognizes the text located in an image
!----------------------------------------------------------!

cOCR.OCR                        PROCEDURE(STRING psInputFile,LONG plhDIB,UNSIGNED punPageNo) !! ,BYTE

eMethodName                     EQUATE('.OCR ')

bErrorLevel                     BYTE
bIdx                            BYTE               ! index counter
bImageCount                     BYTE               ! image count in multi-image files

lRetVal                         LONG

  CODE

  SELF.Trace(eMethodName)

  LOOP 1 TIMES

    bErrorLevel  = LEVEL:Benign
    lProgressFEq = SELF.lProgressFEq
    szOutputText = eOCRNil

    IF psInputFile ~= eOCRNil

      SELF.szInputFile = CLIP(psInputFile)
      SELF.Trace(eMethodName & 'szInputFile = ' & SELF.szInputFile)

      IF SELF.bLoadSingleImage = True
        IF SELF.LoadImage(psInputFile) ~= LEVEL:Benign THEN BREAK.
      ELSE
        IF SELF.LoadMultipleImages(psInputFile) ~= LEVEL:Benign THEN BREAK.
      END ! if
    ELSIF plhDIB ~= eOCRZero
      SELF.Trace(eMethodName & 'plhDIB = ' & plhDIB)
      SELF.bLoadSingleImage = True
      IF SELF.DIBToImage(plhDIB) ~= LEVEL:Benign THEN BREAK.
    ELSE
      SELF.Trace(eMethodName & 'no input file or DIB handle passed to function.')
      BREAK
    END ! if

    SELF.SetOutputMode(eOM_Text)
    SELF.SetLanguage()
    SELF.lOutputHandlerAddress = ADDRESS(OutputHandler)
    SELF.lOutputHandler        = OCRSetOutputHandler(SELF.lOutputHandlerAddress)

    SELF.Trace(eMethodName & 'lOutputHandlerAddress = ' & SELF.lOutputHandlerAddress)
    SELF.Trace(eMethodName & 'lOutputHandler = '        & SELF.lOutputHandler)

    IF SELF.lProgressFEq ~= eOCRZero
      SELF.lProgressHandlerAddress = ADDRESS(ProgressHandler)
      SELF.lProgressHandler        = OCRSetProgressHandler(SELF.lProgressHandlerAddress)
      SELF.Trace(eMethodName & 'lProgressHandlerAddress = ' & SELF.lProgressHandlerAddress)
      SELF.Trace(eMethodName & 'lProgressHandler = '        & SELF.lProgressHandler)
    END ! if

    SELF.lNoisy = SELF.bReduceNoise

    SELF.Trace(eMethodName & 'lNoisy = ' & SELF.lNoisy)

    SETCURSOR(CURSOR:Wait)

    IF SELF.bLoadSingleImage = True
      DO OCRImage
      IF bErrorLevel ~= LEVEL:Benign THEN BREAK.
    ELSE

      bImageCount = GetNbImages(SELF.lSet)

      SELF.Trace(eMethodName & 'Image Count = ' & bImageCount)
      SELF.Trace(eMethodName & 'Page No. = '    & punPageNo)

      IF punPageNo ~= eOCRZero
        IF NOT INRANGE(punPageNo - eOCROne,eOCRZero,bImageCount)
          punPageNo = eOCRZero
          SELF.Trace(eMethodName & 'Page No. changed to = '    & punPageNo & ' - (All Pages Flag)')
        END ! if
      END ! if

      IF punPageNo = eOCRZero ! all pages
        LOOP bIdx = eOCRZero TO bImageCount - eOCROne
          SELF.lImg = GetImage(SELF.lSet,bIdx)
          IF SELF.lImg ~= eOCRZero
            DO OCRImage
            IF bErrorLevel ~= LEVEL:Benign THEN BREAK.
          END ! if
        END ! loop
      ELSE
        SELF.lImg = GetImage(SELF.lSet,punPageNo - eOCROne)
        IF SELF.lImg ~= eOCRZero
          DO OCRImage
          IF bErrorLevel ~= LEVEL:Benign THEN BREAK.
        END ! if
      END ! if
    END ! if

    IF szOutputText ~= eOCRNil

      szOutputText = szOutputText[eOCROne : LEN(szOutputText) - 2]

      IF SELF.lOutputTextFEq ~= eOCRZero
        CHANGE(SELF.lOutputTextFEq,szOutputText)
      END ! if

      IF SELF.bOutputToClipboard = True
        SETCLIPBOARD(szOutputText)
      END ! if
    ELSE
      SELF.SetLastError(eUnableToOCRImageErr,eUnableToOCRImageErrDesc)
      bErrorLevel = LEVEL:Fatal
    END ! if

    IF SELF.bLoadSingleImage = True
      SELF.FreeImage()
    ELSE
      SELF.FreeMultipleImages()
    END ! if
  END ! loop

  SETCURSOR()

  SELF.Trace(eMethodName & 'Complete Returned = ' & bErrorLevel)

  RETURN bErrorLevel

!-------------------------------------!

OCRImage                        ROUTINE

  LOOP 1 TIMES
    DO CleanUpImage
    IF bErrorLevel ~= LEVEL:Benign THEN BREAK.

    lRetVal = OCR(SELF.lImg,SELF.lNoisy)
    IF lRetVal ~= eOCRZero
      SELF.SetLastError(eOCRErr,eOCRErrDesc)
      bErrorLevel = LEVEL:Fatal
    END ! if
    SELF.Trace(eMethodName & 'OCR Image - lRetVal = ' & lRetVal)
  END ! loop

!-------------------------------------!

CleanupImage                    ROUTINE

 LOOP 1 TIMES
   bErrorLevel = SELF.DeskewImage()
   IF bErrorLevel ~= LEVEL:Benign THEN BREAK.

   bErrorLevel = SELF.EraseBlackBorders()
   IF bErrorLevel ~= LEVEL:Benign THEN BREAK.
 END ! loop

!----------------------------------------------------------!
! DisplayImageInfo PROCEDURE(STRING psInputFile),BYTE
!
! Display image info
!----------------------------------------------------------!

cOCR.DisplayImageInfo           PROCEDURE(STRING psInputFile) !! ,BYTE

eMethodName                     EQUATE('.DisplayImageInfo ')

bErrorLevel                     BYTE
bImageCount                     BYTE                       ! image count in multi-image files

lHeight                         LONG
lHRes                           LONG
lVRes                           LONG
lWidth                          LONG

szImageFileName                 CSTRING(101)

  CODE

  LOOP 1 TIMES

    SELF.Trace(eMethodName)

    szImageFileName = CLIP(SELF.ExtractNameFromPathname(psInputFile,True))

    IF SELF.bLoadSingleImage = True
      bErrorLevel = SELF.LoadImage(psInputFile)
      IF bErrorLevel ~= LEVEL:Benign THEN BREAK.

      GetImgRes (SELF.lImg,lHRes ,lVRes  )
      GetImgSize(SELF.lImg,lWidth,lHeight)

      MESSAGE('Name: '       & szImageFileName           & '||' & |
              'Resolution: ' & lHRes  & ' x ' & lVRes    & '||' & |
              'Size: '       & lWidth & ' x ' & lHeight,          |
              'Image Info',ICON:Hand)
      SELF.Trace(eMethodName & 'Image Name = '       & szImageFileName)
      SELF.Trace(eMethodName & 'Image Resolution = ' & lHRes  & ' x ' & lVRes)
      SELF.Trace(eMethodName & 'Image Size = '       & lWidth & ' x ' & lHeight)
      SELF.FreeImage()

    ELSE

      bErrorLevel = SELF.LoadMultipleImages(psInputFile)
      IF bErrorLevel ~= LEVEL:Benign THEN BREAK.

      bImageCount = GetNbImages(SELF.lSet)
      SELF.lImg   = GetImage(SELF.lSet,eOCRZero)

      IF SELF.lImg ~= eOCRZero
        GetImgRes (SELF.lImg,lHRes ,lVRes  )
        GetImgSize(SELF.lImg,lWidth,lHeight)

        MESSAGE('Name: '        & szImageFileName           & '||' & |
                'Image Count: ' & bImageCount               & '||' & |
                'Resolution: '  & lHRes  & ' x ' & lVRes    & '||' & |
                'Size: '        & lWidth & ' x ' & lHeight,                                     |
                'Image Info',ICON:Hand)
        SELF.Trace(eMethodName & 'Image Name = '       & szImageFileName)
        SELF.Trace(eMethodName & 'Image Count = '      & bImageCount)
        SELF.Trace(eMethodName & 'Image Resolution = ' & lHRes  & ' x ' & lVRes)
        SELF.Trace(eMethodName & 'Image Size = '       & lWidth & ' x ' & lHeight)
        SELF.FreeMultipleImages()
      END ! if
    END ! if
  END ! if

  SELF.Trace(eMethodName & 'Complete Returned = ' & bErrorLevel)

  RETURN bErrorLevel

!----------------------------------------------------------!
! OutputDebugString(STRING psDebugString),PRIVATE
!
! Output trace/debug information
!----------------------------------------------------------!

cOCR.OutputDebugString          PROCEDURE(STRING psDebugString) !! ,PRIVATE

usStringLen                     USHORT

szDebugString                   &CSTRING

  CODE

  usStringLen    = LEN(CLIP(psDebugString)) + eOCROne
  szDebugString &= NEW(CSTRING(usStringLen))
  szDebugString  = CLIP(psDebugString)

  OutputDebugString(szDebugString)

  DISPOSE(szDebugString)

!----------------------------------------------------------!
! Trace PROCEDURE(STRING psDbgMessage, <SIGNED piEvent>)
!
! Trace Function
!----------------------------------------------------------!

cOCR.Trace                      PROCEDURE(STRING psDebugMessage)

eClassName                      EQUATE('cOCR')

lMaxMsgLen                      LONG
lMsgLen                         LONG
lMsgPtr                         LONG

sMessage                        STRING(eOCRTraceMaxMsgLen)
sOutPut                         STRING(eOCRTraceOutputLen)

  CODE

  IF ~SELF.bTraceOn THEN RETURN.

  lMaxMsgLen = eOCRTraceOutputLen
  sMessage   = eClassName & CLIP(psDebugMessage)
  lMsgLen    = LEN(CLIP(sMessage))

  LOOP
    IF lMsgLen <= lMaxMsgLen
      sMessage = CLIP(sMessage) & '<13><10>'
      SELF.OutPutDebugString(sMessage)
      BREAK
    ELSE
      sOutPut = sMessage[eOCROne : lMaxMsgLen] & '<13><10>'
      SELF.OutPutDebugString(sOutPut)
      lMsgPtr  = lMaxMsgLen + 1
      sMessage = eClassName &  ' ' & sMessage[lMsgPtr : lMsgPtr + lMsgLen]
      lMsgLen  = LEN(CLIP(sMessage))
    END ! if
  END ! loop

!----------------------------------------------------------!
! AddBackSlash(STRING psPathInfo),STRING
!
! Add Backslash to the end of passed path
!----------------------------------------------------------!

cOCR.AddBackSlash               PROCEDURE(STRING psPathInfo) !! ,STRING

sResult                         STRING(256)

  CODE

  sResult = psPathInfo

  IF sResult ~= ''
    IF ~(sResult[LEN(CLIP(sResult))] = '\')
      sResult = CLIP(sResult ) & '\'
    END ! if
  END ! if

  RETURN sResult

!----------------------------------------------------------!
! AppCompletePathName(),STRING
!
! Return the application path and name
!----------------------------------------------------------!

cOCR.AppCompletePathName        PROCEDURE()                !! ,STRING

lFNSize                         LONG

unInstance                      UNSIGNED

szBuffer                        CSTRING(256)

sComplPathName                  STRING(256)

  CODE

  unInstance     = SYSTEM{PROP:AppInstance}
  sComplPathName = ''
  szBuffer       = ''
  lFNSize        = (SIZE(szBuffer) - 1)

  Junk#          = GetModuleFileName(unInstance,szBuffer,lFNSize)
  sComplPathName = szBuffer

  RETURN LONGPATH(sComplPathName)

!----------------------------------------------------------!
! ProgramDir(BYTE pbAddBackSlash = False),STRING
!
! Return the application directory
!----------------------------------------------------------!

cOCR.ProgramDir                 PROCEDURE(BYTE pbAddBackSlash = False) !! ,STRING

sAppDir                         STRING(256)
sBuffer                         STRING(256)

  CODE

  sBuffer = SELF.ExtractDirFromPathname(SELF.AppCompletePathName())

  IF pbAddBackSlash
    sAppDir = SELF.AddBackSlash(LONGPATH(sBuffer))
  ELSE
    sAppDir = SELF.RemoveBackSlash(LONGPATH(sBuffer))
  END ! if

  RETURN sAppDir

!----------------------------------------------------------!
! RemoveBackSlash(STRING psPathInfo),STRING
!
! Remove the backslash from the end of the passed path
!----------------------------------------------------------!

cOCR.RemoveBackSlash            PROCEDURE(STRING psPathInfo) !! ,STRING

sResult                         STRING(256)

  CODE

  sResult = psPathInfo

  IF sResult[LEN(CLIP(sResult))] = '\'
    sResult = SUB(sResult,1,LEN(CLIP(sResult)) - 1)
  END ! if

  RETURN sResult

!----------------------------------------------------------!
! ExtractNameFromPathname(STRING psPathInfo,BYTE pbWithExtension = False),STRING
!
! Extract the file name from the passed path
!----------------------------------------------------------!

cOCR.ExtractNameFromPathname    PROCEDURE(STRING psPathInfo,BYTE pbWithExtension = False) !! ,STRING

iIdx                            SHORT

sBuffer                         STRING(256)
sResult                         STRING(256)

  CODE

  sBuffer = psPathInfo

  LOOP iIdx = LEN(CLIP(sBuffer)) TO 1 BY -1
    IF sBuffer[iIdx] = '\'
      BREAK
    END ! if
  END ! loop
  

  IF iIdx > 0
    sResult = SUB(sBuffer,iIdx + 1,LEN(CLIP(sBuffer))-(iIdx))
    IF ~pbWithExtension
      sBuffer = sResult
      LOOP iIdx = LEN(CLIP(sBuffer)) TO 1 BY -1
        IF sBuffer[iIdx] = '.'
          BREAK
        END ! if
      END ! loop
      IF iIdx > 0
        sResult = SUB(sBuffer,1,iIdx - 1)
      END ! if
    END ! if
  ELSE
    sResult = ''
  END ! if

  RETURN sResult

!----------------------------------------------------------!
! ExtractDirFromPathname(STRING psPathInfo,BYTE pbAddBackSlash = False),STRING
!
! Extract the directory name from the passed path
!----------------------------------------------------------!

cOCR.ExtractDirFromPathname     PROCEDURE(STRING psPathInfo,BYTE pbAddBackSlash = False) !! ,STRING

iIdx                            SHORT

sBuffer                         STRING(256)
sResult                         STRING(256)

  CODE
  sBuffer = psPathInfo
  LOOP iIdx = LEN(CLIP(sBuffer)) TO 1 BY -1
    IF sBuffer[iIdx] = '\' THEN BREAK.
  END ! loop

  IF iIdx > 0
    IF pbAddBackSlash
      sResult = SELF.AddBackSlash(SUB(sBuffer,1,iIdx))
    ELSE
      sResult = SELF.RemoveBackSlash(SUB(sBuffer,1,iIdx))
    END ! if
  ELSE
    sResult = ''
  END ! if

  RETURN LONGPATH(sResult)

!----------------------------------------------------------!
!----------------------------------------------------------!

! non class/local procedures

!----------------------------------------------------------!
! PROCEDURE(LONG plInfotype,LONG plParam),PASCAL
!
! Output handler
!----------------------------------------------------------!

OutputHandler                   PROCEDURE(LONG plInfotype,LONG plParam)

  CODE

  CASE plInfoType
  OF eOT_TEXT
    szOutputText = szOutputText & CHR(plParam)
  OF eOT_ENDL
    szOutputText = szOutputText & '<13,10>'
  OF eOT_ENDZ
    szOutputText = szOutputText & '<13,10>'
  OF eOT_PROP
  OF eOT_ITAL
  OF eOT_BOLD
  OF eOT_UNDS
  OF eOT_SIZE
  OF eOT_HILT
  OF eOT_BITM
  ELSE
    MESSAGE('Unknown plInfoType: ' & plInfoType)
  END ! case

!----------------------------------------------------------!
! ProgressHandler PROCEDURE(LONG plProgress),PASCAL
!
! Progress handler
!----------------------------------------------------------!

ProgressHandler                 PROCEDURE(LONG plProgress)

lOCRComplete                    LONG

  CODE

  CHANGE(lProgressFEq,plProgress)

  IF plProgress = 100
    lOCRComplete = True
  ELSE
    lOCRComplete = False
  END ! if

  RETURN lOCRComplete

!----------------------------------------------------------!

